@@ -32,13 +32,15 @@
 
 QTEST_MAIN(KCompressionDeviceTest)
 
+static QString archiveFileName(const QString &extension)
+{
+    return QFINDTESTDATA(QString("kcompressiondevice_test.%1").arg(extension));
+}
+
 QNetworkReply *KCompressionDeviceTest::getArchive(const QString &extension)
 {
-    const QString kcompressionTest = QString("kcompressiondevice_test.%1").arg(extension);
-    QNetworkReply *r = qnam.get(
-                QNetworkRequest(
-                    QUrl::fromLocalFile(
-                        QFINDTESTDATA(kcompressionTest))));
+    const QString kcompressionTest = archiveFileName(extension);
+    QNetworkReply *r = qnam.get(QNetworkRequest(QUrl::fromLocalFile(kcompressionTest)));
 
     QEventLoop l;
     connect(&qnam, &QNetworkAccessManager::finished, &l, &QEventLoop::quit);
@@ -56,9 +58,10 @@ QString KCompressionDeviceTest::formatExtension(KCompressionDevice::CompressionT
         return "tar.bz2";
     case KCompressionDevice::Xz:
         return "tar.xz";
-    default:
+    case KCompressionDevice::None:
         return QString();
     }
+    return QString(); // silence compiler warning
 }
 
 void KCompressionDeviceTest::setDeviceToArchive(
@@ -75,6 +78,9 @@ void KCompressionDeviceTest::testBufferedDevice(KCompressionDevice::CompressionT
     QNetworkReply *r = getArchive(formatExtension(type));
     const QByteArray data = r->readAll();
     QVERIFY(!data.isEmpty());
+    const int expectedSize = QFileInfo(archiveFileName(formatExtension(type))).size();
+    QVERIFY(expectedSize > 0);
+    QCOMPARE(data.size(), expectedSize);
     QBuffer *b = new QBuffer;
     b->setData(data);
 
@@ -85,6 +91,7 @@ void KCompressionDeviceTest::testBufferedDevice(KCompressionDevice::CompressionT
 void KCompressionDeviceTest::testExtraction()
 {
     QTemporaryDir temp;
+    QString oldCurrentDir = QDir::currentPath();
     QDir::setCurrent(temp.path());
 
     QVERIFY(archive->open(QIODevice::ReadOnly));
@@ -107,13 +114,14 @@ void KCompressionDeviceTest::testExtraction()
             << QLatin1String("examples/unzipper/CMakeLists.txt")
             << QLatin1String("examples/unzipper/main.cpp");
 
-    foreach (const QString s, fileList) {
+    for (const QString& s : qAsConst(fileList)) {
         QFileInfo extractedFile(s);
         QFileInfo sourceFile(QFINDTESTDATA("../" + s));
 
         QVERIFY(extractedFile.exists());
         QCOMPARE(extractedFile.size(), sourceFile.size());
     }
+    QDir::setCurrent(oldCurrentDir);
 }
 
 void KCompressionDeviceTest::regularKTarUsage()
@@ -146,3 +154,39 @@ void KCompressionDeviceTest::testXzBufferedDevice()
     QSKIP("This test needs xz support");
 #endif
 }
+
+void KCompressionDeviceTest::testWriteErrorOnOpen()
+{
+    // GIVEN
+    QString fileName("/I/dont/exist/kcompressiondevicetest-write.gz");
+    KCompressionDevice dev(fileName, KCompressionDevice::GZip);
+    // WHEN
+    QVERIFY(!dev.open(QIODevice::WriteOnly));
+    // THEN
+    QCOMPARE(dev.error(), QFileDevice::OpenError);
+#ifdef Q_OS_WIN
+    QCOMPARE(dev.errorString(), QStringLiteral("The system cannot find the path specified."));
+#else
+    QCOMPARE(dev.errorString(), QStringLiteral("No such file or directory"));
+#endif
+}
+
+void KCompressionDeviceTest::testWriteErrorOnClose()
+{
+    // GIVEN
+    QFile file("kcompressiondevicetest-write.gz");
+    KCompressionDevice dev(&file, false, KCompressionDevice::GZip);
+    QVERIFY(dev.open(QIODevice::WriteOnly));
+    const QByteArray data = "Hello world";
+    QCOMPARE(dev.write(data), data.size());
+    // This is nasty, it's just a way to try and trigger an error on flush, without filling up a partition first ;)
+    file.close();
+    QVERIFY(file.open(QIODevice::ReadOnly));
+    QTest::ignoreMessage(QtWarningMsg, "QIODevice::write (QFile, \"kcompressiondevicetest-write.gz\"): ReadOnly device");
+
+    // WHEN
+    dev.close(); // I want a QVERIFY here... https://bugreports.qt.io/browse/QTBUG-70033
+
+    // THEN
+    QCOMPARE(int(dev.error()), int(QFileDevice::WriteError));
+}
